@@ -135,6 +135,7 @@ class tabsqlitedb:
         self.old_phrases = []
         self.filename = filename
         self._user_db = user_db
+        self.reset_phrases_cache()
 
         if create_database or os.path.isfile(self.filename):
             self.db = sqlite3.connect(self.filename)
@@ -218,6 +219,7 @@ class tabsqlitedb:
             default_properties=self._default_ime_attributes)
         # shared variables in this class:
         self._mlen = int(self.ime_properties.get("max_key_length"))
+        self._snum = self.ime_properties.get("serial_number")
         self._is_chinese = self.is_chinese()
         self._is_cjk = self.is_cjk()
         self.user_can_define_phrase = self.ime_properties.get(
@@ -249,6 +251,12 @@ class tabsqlitedb:
         self.possible_tabkeys_lengths = self.get_possible_tabkeys_lengths()
         self.startchars = self.get_start_chars ()
 
+        import ibus_table_location
+        tables_path = path.join(ibus_table_location.data_home(),  "tables")
+        cache_name = os.path.basename(self.filename).replace('.db', '.cache')
+        self.cache_path = path.join(tables_path, cache_name)
+        self.load_phrases_cache()
+
         if not user_db or create_database:
             # No user database requested or we are
             # just creating the system database and
@@ -267,8 +275,6 @@ class tabsqlitedb:
             #
             # “HOME=/foobar ibus-table-createdb” should not fail if
             # “/foobar” is not writeable.
-            import ibus_table_location
-            tables_path = path.join(ibus_table_location.data_home(),  "tables")
             if not path.isdir(tables_path):
                 old_tables_path = os.path.join(
                     os.getenv('HOME'), '.ibus/tables')
@@ -447,6 +453,7 @@ class tabsqlitedb:
             self.db.execute(sqlstr, sqlargs)
             if commit:
                 self.db.commit()
+            self.invalidate_phrases_cache(tabkeys)
         except:
             import traceback
             traceback.print_exc()
@@ -455,11 +462,50 @@ class tabsqlitedb:
         '''
         Trigger a checkpoint operation.
         '''
+        self.save_phrases_cache()
         if self._user_db is None:
             return
         self.db.commit()
         self.db.execute('PRAGMA wal_checkpoint;')
 
+    def reset_phrases_cache (self):
+        if debug_level > 1:
+            sys.stderr.write("reset_phrases_cache()\n")
+        self._phrases_cache = {}
+
+    def invalidate_phrases_cache (self, tabkeys=u''):
+        if debug_level > 1:
+            sys.stderr.write("invalidate_phrases_cache()\n")
+        for i in range(1, self._mlen + 1):
+            if self._phrases_cache.get(tabkeys[0:i]):
+                self._phrases_cache.pop(tabkeys[0:i])
+
+    def load_phrases_cache(self):
+        if debug_level > 1:
+            sys.stderr.write("load_phrases_cache()\n")
+        try:
+            import json
+            self._phrases_cache = json.load(open(self.cache_path, 'r'))
+            snum = self._phrases_cache.get('serial_number')
+            if not snum or (snum != self._snum):
+                self._phrases_cache = {}
+        except:
+            pass
+
+    def save_phrases_cache(self):
+        if debug_level > 1:
+            sys.stderr.write("save_phrases_cache()\n")
+        try:
+            import json
+            self._phrases_cache['serial_number'] = self._snum
+            _cache_path = self.cache_path + '.tmp'
+            # The system may be break stores on rebooting, so
+            # dump to temporary file and then replace it.
+            json.dump(self._phrases_cache, open(_cache_path, 'w'))
+            os.replace(_cache_path, self.cache_path);
+        except:
+            pass
+
     def is_chinese (self):
         __lang = self.ime_properties.get('languages')
         if __lang:
@@ -666,6 +712,7 @@ class tabsqlitedb:
                 'phrase': phrase,
                 'freq': freq,
                 'user_freq': user_freq})
+            self.invalidate_phrases_cache(tabkeys)
         self.db.executemany(insert_sqlstr, insert_sqlargs)
         self.db.commit()
         self.db.execute('PRAGMA wal_checkpoint;')
@@ -720,6 +767,7 @@ class tabsqlitedb:
             self.db.execute (insert_sqlstr, insert_sqlargs)
             if commit:
                 self.db.commit()
+            self.invalidate_phrases_cache(tabkeys)
         except:
             import traceback
             traceback.print_exc()
@@ -888,21 +936,31 @@ class tabsqlitedb:
         '''
         if not tabkeys:
             return []
+        # query phrases cache first
+        best = self._phrases_cache.get(tabkeys)
+        if best:
+            return best
         one_char_condition = ''
         if onechar:
             # for some users really like to select only single characters
             one_char_condition = ' AND length(phrase)=1 '
 
-        sqlstr = '''
-        SELECT tabkeys, phrase, freq, user_freq FROM
-        (
+        if self.user_can_define_phrase or self.dynamic_adjust:
+            sqlstr = '''
+            SELECT tabkeys, phrase, freq, user_freq FROM
+            (
+                SELECT tabkeys, phrase, freq, user_freq FROM main.phrases
+                WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar %(one_char_condition)s
+                UNION ALL
+                SELECT tabkeys, phrase, freq, user_freq FROM user_db.phrases
+                WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar %(one_char_condition)s
+            )
+            ''' % {'one_char_condition': one_char_condition}
+        else:
+            sqlstr = '''
             SELECT tabkeys, phrase, freq, user_freq FROM main.phrases
             WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar %(one_char_condition)s
-            UNION ALL
-            SELECT tabkeys, phrase, freq, user_freq FROM user_db.phrases
-            WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar %(one_char_condition)s
-        )
-        ''' % {'one_char_condition': one_char_condition}
+            ''' % {'one_char_condition': one_char_condition}
         escapechar = '☺'
         for c in '!@#':
             if c not in [single_wildcard_char, multi_wildcard_char]:
@@ -962,6 +1020,7 @@ class tabsqlitedb:
             chinese_mode=chinese_mode)
         if debug_level > 1:
             sys.stderr.write("select_words() best=%s\n" %repr(best))
+        self._phrases_cache[tabkeys] = best
         return best
 
     def select_chinese_characters_by_pinyin(
@@ -1341,6 +1400,7 @@ class tabsqlitedb:
         self.db.execute(delete_sqlstr, delete_sqlargs)
         if commit:
             self.db.commit()
+        self.invalidate_phrases_cache(tabkeys)
 
     def extract_user_phrases(
             self, database_file='', old_database_version='0.0'):
